--------------------------------------------------------------------------------
-- Company: UNIVERSITY OF CONNECTICUT
-- Engineer: Farrukh Hijaz
--
-- Create Date:    10/07/2014
-- Design Name:    
-- Module Name:    wb_eightcharled - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity wb_eightcharled is
	Port ( clk_i : in std_logic;
          rst_i : in std_logic;
		    adr_i : in std_logic_vector(31 downto 0);
          dat_i : in std_logic_vector(31 downto 0);
          dat_o : out std_logic_vector(31 downto 0);
          ack_o : out std_logic;
          stb_i : in std_logic;
          we_i  : in std_logic;
			 segment : out std_logic_vector(6 downto 0);
          an : out std_logic_vector(3 downto 0);
          dp : out std_logic
		);
end wb_eightcharled;

architecture Behavioral of wb_eightcharled is
	signal char0, char1, char2, char3 : std_logic_vector(7 downto 0);
	signal char4, char5, char6, char7 : std_logic_vector(7 downto 0);
	signal onemsec_clk : std_logic;
begin

 	onemsec_clk_divider : entity work.clock_divider
		generic map ( divisor => 2**16 )
		port map ( clk_in => clk_i, reset => rst_i, clk_out => onemsec_clk );

	-- convert eight characters to the appropriate digit and segment signals
	string2leds : entity work.string2leds
		port map ( char0 => char0, char1 => char1, char2 =>char2, char3 => char3,
					  char4 => char4, char5 => char5, char6 =>char6, char7 => char7,
					  onemsec_clk => onemsec_clk, 
					  sys_rst => rst_i,
				 	  enable => '1',
				     segment => segment,
			 	 	  an => an,
					  dp => dp );

	process( clk_i, rst_i )
		variable ascii_0 : std_logic_vector(7 downto 0);
		variable ascii_1 : std_logic_vector(7 downto 0);
		variable ascii_2 : std_logic_vector(7 downto 0);
		variable ascii_3 : std_logic_vector(7 downto 0);
	begin
		if ( rst_i = '1' ) then
			char7 <= X"20";
			char6 <= X"20";
			char5 <= X"20";
			char4 <= X"20";
			char3 <= X"20";
			char2 <= X"20";
			char1 <= X"20";
			char0 <= X"20";
		elsif ( rising_edge(clk_i) ) then
			if ( stb_i='1' and we_i='1' ) then
				ascii_0:= dat_i(7 downto 0);
				ascii_1:= dat_i(15 downto 8);
				ascii_2:= dat_i(23 downto 16);
				ascii_3:= dat_i(31 downto 24);
				
				if(ascii_3 = X"08" OR ascii_3 = X"7F") then
					char3 <= X"20";
				else
					char3 <= dat_i(31 downto 24);
				end if;
				if(ascii_2 = X"08" OR ascii_2 = X"7F") then
					char2 <= X"20";
				else
					char2 <= dat_i(23 downto 16);
				end if;
				if(ascii_1 = X"08" OR ascii_1 = X"7F") then
					char1 <= X"20";
				else
					char1 <= dat_i(15 downto 8);
				end if;
				if(ascii_0 = X"08" OR ascii_0 = X"7F") then
					char0 <= X"20";
				else
					char0 <= dat_i(7 downto 0);
				end if;
	
			end if;

		end if;

	end process;

	-- always acknowledge writes, since there are no wait delays to be inserted
	ack_o <= '1';

end Behavioral;

